"""
Functions to calculate delta CR, delta VV, delta gamma, and snow index.
"""

import numpy as np
import pandas as pd
import xarray as xr

from typing import Union

import logging
log = logging.getLogger(__name__)

def calc_delta_VV(dataset: xr.Dataset, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Calculate change in VV amplitude between current time step and previous
    from each relative orbit and adds to dataset.
    
    delta-gamma-VV (i, t) = gamma-VV (i, t) - gamma-VV(i, t_previous)

    Returns nans in time slice of deltaVV with no previous image 
    (first of each relative orbit)

    Args:
    dataset: Xarray Dataset of sentinel images
    inplace: operate on dataset in place or return copy

    Returns:
    dataset: Xarray Dataset of sentinel images with 'deltaVV' added as data var
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check for amp
    if 's1_units' in dataset.attrs.keys():
        assert dataset.attrs['s1_units'] == 'dB', 'Sentinel-1 units must be in dB'

    # get all unique relative orbits
    orbits = np.unique(dataset['relative_orbit'].values)

    for orbit in orbits:
        
        # Calculate change in gamma-VV between previous and current time step from the same relative orbit
        diffVV = dataset['s1'].sel(time = dataset.relative_orbit == orbit, band = 'VV').diff(dim = 'time')
        
        # if adding new 
        if 'deltaVV' not in dataset.data_vars:
            # add delta-gamma-VV as variable to dataset
            dataset['deltaVV'] = diffVV
        
        else:
            # update deltaVV times with this.
            dataset = dataset.copy(deep=True)
            dataset['deltaVV'].loc[dict(time = diffVV.time)] = diffVV
    
    if not inplace:
        return dataset

def calc_delta_cross_ratio(dataset: xr.Dataset, A: float = 2, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Calculate change in cross-polarization ratio for all time steps.
    
    delta-gamma-cr (i, t) = gamma-cr (i, t) - gamma-cr (i, t_previous)

    with:
    gamma-cr = A * VH - VV

    and gamma-VH and gamma-VV in dB. Lieven's et al. 2021 tests A over [1, 2, 3]
    and fit to A = 2.

    Returns nans in time slice of deltaCR with no previous image 
    (first of each relative orbit)

    Args:
    dataset: Xarray Dataset of sentinel images
    A: fitting parameter [default = 2]
    inplace: operate on dataset in place or return copy

    Returns:
    dataset: Xarray Dataset of sentinel images with deltaCR added as data var
    """

    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check for amp
    if 's1_units' in dataset.attrs.keys():
        assert dataset.attrs['s1_units'] == 'dB', 'Sentinel-1 units must be in dB'

    # calculate cross ratio of VH to VV with fitting parameter A
    gamma_cr = (A * dataset['s1'].sel(band='VH')) - dataset['s1'].sel(band='VV')

    # calculate cross ratio of VH to VV without fitting parameter A
    gamma_crt = dataset['s1'].sel(band='VH') - dataset['s1'].sel(band='VV')  # db

    # get all unique relative orbits
    orbits = np.unique(dataset['relative_orbit'].values)
    
    # Identify previous image from the same relative orbit (6, 12, 18, or 24 days ago)
    for orbit in orbits:

        # Calculate change in gamma-cr between previous and current time step
        diffCR = gamma_cr.sel(time = dataset.relative_orbit == orbit).diff(dim = 'time')
        diffCRt = gamma_crt.sel(time = dataset.relative_orbit == orbit).diff(dim = 'time')
        # add delta-gamma-cr as band to dataset
        # if adding new 
        if 'deltaCR' not in dataset.data_vars:
            # add delta-gamma-CR as new variable to dataset
            dataset['deltaCR'] = diffCR
            dataset['deltaCRt'] = diffCRt
        else:
            # update deltaCR times with this.
            dataset = dataset.copy(deep=True)
            dataset['deltaCR'].loc[dict(time=diffCR.time)] = diffCR
            dataset['deltaCRt'].loc[dict(time=diffCRt.time)] = diffCRt
    if not inplace:
        return dataset

def calc_delta_gamma(dataset: xr.Dataset, B: float = 0.5, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Calculate change in combined gamma parameter (VV and cross-ratio) between 
    current time step and previous.
    
    delta-gamma(i, t) = (1 - FCF) * delta-gamma-CR (i, t) + (FCF) * B * gamma-VV(i, t)

    with:
    FCF - being the PROBA-V fraction forest cover (ranging from 0 to 1).
    B - fitting parameter tested over 0 to 1 in steps of 0.1 and optimized at 0.5.

    Args:
    dataset: Xarray Dataset of sentinel images with delta-gamma-cr and delta-gamma-VV

    Returns:
    dataset: Xarray Dataset of sentinel images with delta-gamma added as band
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # check to ensure fcf is 0-1 not 0-100
    assert dataset['fcf'].max() <= 1, "Forest cover fraction must be scaled 0-1"
    assert dataset['fcf'].min() >= 0, "Forest cover fraction must be scaled 0-1"

    # Calculate delta gamma from delta-gamma-cr, delta-gamma-VV and FCF
    # add delta-gamma as band to dataset
    dataset['deltaGamma'] = (1 - dataset['fcf']) * dataset['deltaCR'] + \
        (dataset['fcf'] * B * dataset['deltaVV'])

    if not inplace:
        return dataset

def clip_delta_gamma_outlier(dataset: xr.Dataset, thresh: float = 3, inplace: bool = False) -> Union[None, xr.Dataset]: 
    """
    Clip delta gamma to -3 -> 3 dB

    Args:
    dataset: Xarray Dataset of sentinel images with delta-gamma
    thresh: threshold in dB to clip delta Gamma to

    Returns:
    dataset: Xarray Dataset of sentinel images with delta gamma changes clipped
    to -3 -> 3
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # change values above 3 and not nan to 3
    dataset['deltaGamma'] = dataset['deltaGamma'].where((dataset['deltaGamma'] < thresh) | dataset['deltaGamma'].isnull(), thresh)
    
    # change values below -3 and not nan to -3
    dataset['deltaGamma'] = dataset['deltaGamma'].where((dataset['deltaGamma'] > -thresh) | dataset['deltaGamma'].isnull(), -thresh)
    
    if not inplace:
        return dataset

def find_repeat_interval(dataset: xr.Dataset) -> pd.Timedelta:
    """
    Figures out if datasets repeat interval is 6 days or 12 days. Should raise error
    if not multuple of 6 days.

    Args:
    dataset: dataset of sentinel-1 images

    Returns:
    repeat: pandas timedelta and number of days between images
    """
    # figure out if 6 or 12 days repeat
    orbit_times = dataset.sel(time = dataset.relative_orbit == dataset['relative_orbit'][0]).time.diff(dim = 'time').values
    repeat = np.nanmedian([pd.Timedelta(i).round('D') for i in orbit_times]).round('D')

    assert repeat.days % 6 == 0, f"Calculated repeat interval, {repeat}, is not multiple of 6 days."

    return repeat

def calc_prev_snow_index(dataset: xr.Dataset, current_time: np.datetime64, repeat: pd.Timedelta) -> xr.DataArray:
    """
    Calculate previous snow index for +/- 5 days (6 day timestep) or +/- 11 days 
    (12 day time step) from previous time step (6/12 days)'s snow index

    SI (i, t_previous) = sum (t_pri - 5/11 days, t_pri + 5/11 days)(SI * weights) / sum(weights)

    with:
        w_k: as the inverse distance in time from t_previous so for 6-days: 
        wgts=repmat(win+1-abs([-win:win]),dim,1); [1, 2, 3, 4, 5, 6, 5, 4, 3, 2, 1]

    Args:
    dataset: dataset of sentinel-1 images with 'snow-index' data variable
    current_time: the current image date
    repeat: is this region capturing s1 images every 6 or 12 days

    Returns:
    prev_si: the weighted average of previous snow indexes
    """
    # calculate how many days ago we are centering previous snow indexes (6 or 12 days)
    t_prev = current_time - repeat
    # get slice of +- 5 or +- 11 days depending on repeat interval
    t_oldest, t_youngest = pd.to_datetime(t_prev - (repeat - pd.Timedelta('1 day'))) , pd.to_datetime(t_prev + (repeat - pd.Timedelta('1 day')))
    # slice dataset to get all images in previous period
    prev = dataset.sel(time = slice(t_oldest, t_youngest))

    # calculate weights based on days between centered date and image acquistions
    day_weights = repeat.days - np.abs([int((t - t_prev).days) for t in prev.time.values])
    wts = xr.ones_like(prev['snow_index']) * day_weights

    # set wts to nan where snow_index is nan
    wts = wts.where(~prev['snow_index'].isnull())

    # calculate previous snow index weighted by time from last acquistion
    prev_si = (prev['snow_index'] * wts).sum(dim = 'time', skipna = True) / wts.sum(dim = 'time', skipna = True)
    # this is weird but works
    # (prev['snow_index']*wts).sum(dim = 'time') treats nans as 0 so they don't
    # propogate through time series (maybe a problem later for wet snow masking)

    return prev_si

def calc_snow_index(dataset: xr.Dataset, ims_masking: bool = True, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Calculate snow index for each time step from previous time steps' snow index
    weights, and current delta-gamma.

    SI (i, t) = SI (i, t_previous) + delta-gamma (i, t)

    with SI (i, t_previous) as:
        SI (i, t_previous) = sum (t_pri - 5/11 days, t_pri + 5/11 days)(SI * weights) / sum(weights)

    Args:
    dataset: Xarray Dataset of sentinel images with delta-gamma
    ims_masking: whether to mask pixels with the IMS data
    inplace: operate on dataset in place or return copy

    Returns:
    dataset: Xarray Dataset of sentinel images with snow-index added as band
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)

    # set all snow index to 0 to start
    dataset['snow_index'] = xr.zeros_like(dataset['deltaGamma'])

    # find repeat interval of dataset
    repeat = find_repeat_interval(dataset)

    # iterate through time steps
    for ct in dataset.time.values:

        # calculate previous snow index
        prev_si = calc_prev_snow_index(dataset, ct, repeat)

        # set prev_si to 0 at nans
        prev_si = prev_si.where(~prev_si.isnull(), 0)
        
        # add deltaGamma to previous snow index
        dataset['snow_index'].loc[dict(time = ct)] = prev_si + dataset['deltaGamma'].sel(time = ct)
        
        # change to 0 when ims snow cover is not 4
        if ims_masking:
            dataset['snow_index'].loc[dict(time = ct)] = dataset['snow_index'].sel(time = ct).where(dataset['ims'].sel(time = ct) == 4, 0)

        # change to 0 when snow_index is negative
        dataset['snow_index'].loc[dict(time = ct)] = \
            dataset['snow_index'].sel(time = ct).where((dataset['snow_index'].sel(time = ct).isnull()) | (dataset['snow_index'].sel(time = ct) > 0), 0)
    
    if not inplace:
        return dataset


def calc_snow_index_to_snow_depth(dataset: xr.Dataset, C: float = 0.44, inplace: bool = False) -> Union[None, xr.Dataset]:
    """
    Convert current snow-index to snow depth using the C parameter. Varied 
    from [0->1 by 0.01].

    Args:
    dataset: Xarray Dataset of sentinel images with snow index
    C: fitting parameter
    inplace: operate on dataset in place or return copy

    Returns:
    dataset: Xarray Dataset of sentinel images with retrieved snow depth
    """
    # check inplace flag
    if not inplace:
        dataset = dataset.copy(deep=True)
    
    dataset['snow_depth'] = dataset['snow_index'] * C

    if not inplace:
        return dataset


import json

import hyp3_sdk


def download_files_in_jobs(jobname, filelist=None, out_dir='.'):

    hyp3 = hyp3_sdk.HyP3('https://hyp3-api.asf.alaska.edu', prompt=True)

    jobs = hyp3.find_jobs(name=jobname)

    if len(jobs) == 0:
        return

    if filelist:
        with open(filelist) as file:
            lines = [line.rstrip() for line in file]

        # if job has been downloaded, skip it

        for job in jobs:

            if job.files[0]['filename'] in lines:
                continue
            else:
                job.download_files(location=out_dir, create=True)
    else:
        jobs.download_files(location=out_dir, create=True)

    return


if __name__ == '__main__':

    jobname = 'jz_snow_1'

    filelist = "/media/jiangzhu/data1/crrel/sentinel1_20201116_20210327/flist"

    out_dir = "/media/jiangzhu/data1/crrel/sentinel1_20201116_20210327"

    download_files_in_jobs(jobname, filelist=filelist, out_dir=out_dir)

    print('completed ...')



from pathlib import Path

from asf_snow.spicy_snow.download.snowex_lidar import download_dem, download_snow_depth,\
      download_veg_height, make_site_ds

lidar_dir = Path('/home/jiangzhu/data/crrel/lidar_via_zach')

def download_lidar_data(lidar_dir):
      download_snow_depth(lidar_dir)
      download_veg_height(lidar_dir)
      download_dem(lidar_dir)




import geopandas as gpd
import pandas as pd
import numpy as np
from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser

def read_snex_csv(file, col_names=None, skiprows=0, comment="#", numeric_cols=None):
    # read csv file into a dataframe
    df = pd.read_csv(file, names=col_names, comment=comment)

    # drop the first skiprows
    df.drop(df.index[range(0, skiprows)], axis=0, inplace=True)

    # change numeric columns to float datatype
    # numeric_cols = ['Easting', 'Northing', 'Latitude', 'Longitude', 'DensityA', 'DensityB', 'Density',
    #                'SWEA', 'SWEB', 'SWE', 'HS']

    if numeric_cols:
        for col in numeric_cols:
            df[col] = df[col].astype(float)

    # convert df into GeoDataFrame
    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.Longitude, df.Latitude))

    # set CRS
    gdf.crs = 'epsg:4326'

    # write gdf to geojson file
    # gdf.to_file("/home/jiangzhu/data/crrel/snex23_swe/snex.geojson",driver='GeoJSON')
    
    return gdf


def read_snotel(file, lon, lat):

    # /home/jiangzhu/data/crrel/SNOTEL/creamersfield/20220801_20230731.csv
    # Latitude: 64 deg; 52 min N, 64.867 Longitude: 147 deg; 44 min W, -147.733
    # UTC = AKST + 9
    col_names = ['Time', 'snow_depth', 'qc_flag', 'qa_flag', 'snow_depth_collection']

    skiprows = 1

    comment = "#"
    df = pd.read_csv(file, names=col_names, comment=comment)

    # drop the first skiprows
    df.drop(df.index[range(0, skiprows)], axis=0, inplace=True)

    # convert Date string to datetime64 and create the time column
    df['time'] = pd.to_datetime(df['Time'], format='%Y-%m-%d')
    # convert AK time to UTC
    df['time'] = df['time'] + pd.DateOffset(hours=9, minutes=0)
    # convert to float
    df['snow_depth'] = df['snow_depth'].astype(float)
    # convert inch to meter
    df['snow_depth'] = df['snow_depth']*0.0254
    # remove rows with qc_flag='S' or 'N'
    df = df.drop(df[(df['qc_flag'] == 'S') | ( df['qc_flag'] == 'N')].index)
    # convert df into GeoDataFrame
    num = len(df)
    lst_lon = [lon]*num
    lst_lat = [lat]*num

    new_data = {'Longitude': lst_lon, 'Latitude': lst_lat}

    df = df.assign(**new_data)

    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.Longitude, df.Latitude))

    gdf.attrs = {'Location': 'Creamers Field', 'SiteID': 1302, 'County': 'Fairbanks North Star', 'State': 'AK'}

    gdf.crs = 'epsg:4326'

    return gdf


def read_snotel_sd_precip_ratio_temp(file, lon, lat, attrs):

    # /home/jiangzhu/data/crrel/SNOTEL/creamersfield/20220801_20230731.csv
    # Latitude: 64 deg; 52 min N, 64.867 Longitude: 147 deg; 44 min W, -147.733
    # UTC = AKST + 9
    # col_names = ['Time', 'snow_depth', 'qc_flag', 'qa_flag', 'precip_acc', 'precip_inc', 'snow_rain_ratio', 'temp_avg','temp_min']
    # numeric_cols = ['snow_depth', 'precip_acc','precip_inc', 'snow_rain_ratio','temp_avg','temp_min']

    col_names = ['Time', 'snow_depth', 'qc_flag', 'qa_flag', 'precip_acc', 'precip_inc', 'temp_avg','temp_max', 'temp_min', 'snow_density', 'snow_rain_ratio']
    numeric_cols = ['snow_depth', 'precip_acc','precip_inc', 'temp_avg','temp_max', 'temp_min','snow_density', 'snow_rain_ratio']

    skiprows = 1
    comment = "#"
    df = pd.read_csv(file, names=col_names, comment=comment)

    # drop the first skiprows
    df.drop(df.index[range(0, skiprows)], axis=0, inplace=True)

    # convert Date string to datetime64 and create the time column
    df['time'] = pd.to_datetime(df['Time'], format='%Y-%m-%d')
    # convert AK time to UTC
    df['time'] = df['time'] + pd.DateOffset(hours=9, minutes=0)
    # convert to float
    for i in numeric_cols:
        df[i] = df[i].astype(float)

    # convert inch to meter
    df['snow_depth'] = df['snow_depth']*0.0254
    df['precip_inc'] = df['precip_inc']*0.0254
    # remove rows with qc_flag='S' or 'N'
    df = df.drop(df[(df['qc_flag'] == 'S') | ( df['qc_flag'] == 'N')].index)

    # snow_rain_ratio should be in 0 to 100
    ch = df['snow_rain_ratio'] > 100.0
    df.loc[ch, "snow_rain_ratio"] = 100.0

    # convert df into GeoDataFrame
    num = len(df)
    lst_lon = [lon]*num
    lst_lat = [lat]*num

    new_data = {'Longitude': lst_lon, 'Latitude': lst_lat}

    df = df.assign(**new_data)

    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.Longitude, df.Latitude))

    gdf.attrs = attrs

    gdf.crs = 'epsg:4326'

    return gdf

def read_snotel_sd_precip_temp(file, lon, lat, attrs):

    # /home/jiangzhu/data/crrel/SNOTEL/creamersfield/20220801_20230731.csv
    # Latitude: 64 deg; 52 min N, 64.867 Longitude: 147 deg; 44 min W, -147.733
    # UTC = AKST + 9
    col_names = ['Time', 'snow_depth', 'qc_flag', 'qa_flag', 'precip_acc', 'precip_inc', 'temp_avg','temp_max', 'temp_min']
    numeric_cols = ['snow_depth', 'precip_acc','precip_inc','temp_avg','temp_max', 'temp_min']

    skiprows = 1

    comment = "#"
    df = pd.read_csv(file, names=col_names, comment=comment)

    # drop the first skiprows
    df.drop(df.index[range(0, skiprows)], axis=0, inplace=True)

    # convert Date string to datetime64 and create the time column
    df['time'] = pd.to_datetime(df['Time'], format='%Y-%m-%d')
    # convert AK time to UTC
    df['time'] = df['time'] + pd.DateOffset(hours=9, minutes=0)
    # convert to float
    for i in numeric_cols:
        df[i] = df[i].astype(float)

    # convert inch to meter
    df['snow_depth'] = df['snow_depth']*0.0254
    # remove rows with qc_flag='S' or 'N'
    df = df.drop(df[(df['qc_flag'] == 'S') | ( df['qc_flag'] == 'N')].index)
    # convert df into GeoDataFrame
    num = len(df)
    lst_lon = [lon]*num
    lst_lat = [lat]*num

    new_data = {'Longitude': lst_lon, 'Latitude': lst_lat}

    df = df.assign(**new_data)

    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.Longitude, df.Latitude))

    gdf.attrs = attrs

    gdf.crs = 'epsg:4326'

    return gdf




def read_snex23_swe(file):
    """read SNEX23_SWE_*.csv file.
    https://nsidc.org/data/snowex/data?field_data_set_keyword_value=5
    https://nsidc.org/data/snex23_swe/versions/1.
    :param file: SNEX23_SWE_Mar23IOP_AK_20230313_20230316_v01.0.csv
    :return: GeoDataFrame
    """
    col_names = ['State', 'County', 'Site ID', 'Latitude', 'Longitude', 'Elevation', 'Date', 'Depth1', 'Depth2',
                 'Depth3', 'Mass1', 'Mass2', 'Mass3', 'Density1', 'Density2', 'Density3', 'SWE1', 'SWE2', 'SWE3']

    skiprows = 4

    comment = "#"

    numeric_cols = ['Latitude', 'Longitude', 'Elevation', 'Depth1', 'Depth2', 'Depth3', 'Mass1', 'Mass2', 'Mass3',
                    'Density1', 'Density2', 'Density3', 'SWE1', 'SWE2', 'SWE3']
    gdf = read_snex_csv(file, col_names=col_names, skiprows=skiprows, comment=comment, numeric_cols=numeric_cols)

    # convert Date string to datetime64 and create the time column
    gdf['time'] = pd.to_datetime(gdf['Date'], format='%d-%b-%y')
    # convert AK time to UTC
    gdf['time'] = gdf['time'] + pd.DateOffset(hours=9, minutes=0)

    gdf.rename(columns={'Date': 'Time', 'County': 'Location'}, inplace=True)

    # eliminate the bad record in the gdf
    gdf = gdf[(gdf['Depth1'] >= 0.0) & (gdf['Depth2'] >= 0.0) & (gdf['Depth3'] >= 0.0)]

    # calculate the average, snow+depth unit is in meter
    gdf['snow_depth'] = 0.01*(gdf['Depth1'] + gdf['Depth2'] + gdf['Depth3'])/3.0
    gdf['SWE'] = (gdf['SWE1'] + gdf['SWE2'] + gdf['SWE3'])/3.0
    gdf['HS'] = gdf['snow_depth']*100.0
    return gdf


def read_SNEX23_MAR23_SD_AK_20230307_20230316_v01d0(file):

    # SNEX23_MAR23_SD_AK_20230307_20230316_v01.0.csv

    col_names = ['State', 'County', 'Site ID', 'Latitude', 'Longitude', 'Elevation', 'Date', 'Depth1', 'Depth2',
                'Depth3', 'Mass1', 'Mass2', 'Mass3', 'Density1', 'Density2', 'Density3', 'SWE1', 'SWE2', 'SWE3']

    col_names = ['State', 'County', 'Study Area', 'Plot ID', 'ID', 'Date', 'Time', 'Latitude', 'Longitude',
                'Northing', 'Easting', 'Elevation', 'Depth', 'Equipment ID', 'Instrument', 'Version Number']

    skiprows = 2

    comment = "#"

    numeric_cols = ['Latitude', 'Longitude', 'Northing', 'Easting', 'Elevation', 'Depth']

    gdf = read_snex_csv(file, col_names=col_names, skiprows=skiprows, comment=comment, numeric_cols=numeric_cols)

    # eliminate the bad record in the gdf
    gdf = gdf[gdf['Depth'] >= 0.0]
    # snow_depth
    gdf['snow_depth'] = 0.01*gdf['Depth']
    gdf['SWE'] = np.nan
    gdf['HS'] = gdf['Depth']

    return gdf

def read_snex23_snow_water_equivalent(file):
    """read SNEX23_SWE_Mar23IOP_AK_20230313_20230316_v01.0_md.csv
    https://nsidc.org/data/snowex/data?field_data_set_keyword_value=5
    https://nsidc.org/data/snex23_swe/versions/1.
    :param file: SNEX23_SWE_Mar23IOP_AK_20230313_20230316_v01.0.csv
    :return: GeoDataFrame
    """
    col_names = ['State', 'County', 'Site ID', 'Latitude', 'Longitude', 'Elevation', 'Date', 'Depth1', 'Depth2',
                'Depth3', 'Mass1', 'Mass2', 'Mass3', 'Density1', 'Density2', 'Density3', 'SWE1', 'SWE2', 'SWE3']

    skiprows = 2

    comment = "#"

    return read_snex_csv(file, col_names=col_names, skiprows=skiprows, comment=comment)


def read_snex21_ts_summary_swe_v01(file):
    """read the SNEX21_TS_SP_Summary_SWE_v01.csv file
    https://nsidc.org/data/data-access-tool/SNEX21_TS_SP/versions/1
    https://nsidc.org/data/snex21_ts_sp/versions/1
    :param file:SNEX21_TS_SP_Summary_SWE_v01.csv
    :return: GeoDataFrame
    """
    col_names_orig = ['Location', 'Site', 'PitID', 'Date/Local Standard Time', 'UTM Zone', 'Easting (m)',
                      'Northing (m)', 'Latitude', 'Longitude', 'Density A Mean (kg/m^3)', 'Density B Mean (kg/m^3)',
                      'Density Mean (kg/m^3)', 'SWE A (mm)', 'SWE B (mm)', 'SWE (mm)', 'HS (cm)', 'Flag']

    col_names_new = ['Location', 'Site', 'PitID', 'Time', 'UTMZone', 'Easting', 'Northing',
                     'Latitude', 'Longitude', 'DensityA', 'DensityB',
                     'Density', 'SWEA', 'SWEB', 'SWE', 'HS', 'Flag']

    col_rename = {'Date/Local Standard Time': 'Time', 'UTM Zone': 'UTMZone', 'Easting (m)': 'Easting',
                  'Northing (m)': 'Northing', 'Density A Mean (kg/m^3)': 'DensityA',
                  'Density B Mean (kg/m^3)': 'DensityB', 'Density Mean (kg/m^3)':'Density',
                  'SWE A (mm)':'SWEA',  'SWE B (mm)':'SWEB',  'SWE (mm)': 'SWE', 'HS (cm)': 'HS'}


    numeric_cols = ['Easting', 'Northing', 'Latitude', 'Longitude', 'DensityA', 'DensityB', 'Density',
                    'SWEA', 'SWEB', 'SWE', 'HS']

    skiprows = 20

    comment = "#"

    gpd = read_snex_csv(file, col_names=col_names_new, skiprows=skiprows, comment=comment, numeric_cols=numeric_cols)

    # convert Date string to datetime and convert it into UTC (Boise)
    gpd['time'] = pd.to_datetime(gpd['Time'], format='%Y-%m-%dT%H:%M') + pd.DateOffset(hours=6, minutes=0)

    # calculate the snow depth in snex, SWE = hs x ρs/ pw, ps- snow density kg/m3, pw-water density kg/m3 =1000 kg/m3
    # SWE in mm, snex_sd in m, found snex['HS'] is almost equal to snex['snow_depth']

    gpd['snow_depth'] = gpd['SWE']*1000.0/gpd['Density']*0.001

    # gpd['snow_depth'] = gpd['HS']*0.001

    return gpd


def read_snex21_ts_summary_environment_v01(file):
    """read "SNEX21_TS_SP_Summary_Environment_v01.csv"
    :param file: SNEX21_TS_SP_Summary_Environment_v01.csv
    :return: GeoDataFrame
    """

    col_names = ['Location', 'Site', 'PitID', 'Time', 'UTMZone', 'Easting', 'Northing', 'Latitude', 'Longitude',
                 'Precip Type', 'Precip Rate', 'Sky', 'Wind', 'Ground Condition', 'Ground Roughness',
                 'Ground Vegetation', 'Height of Ground Vegetation', 'Canopy']

    numeric_cols = ['Easting', 'Northing', 'Latitude', 'Longitude']

    skiprows = 21

    comment = "#"

    gdf = read_snex_csv(file, col_names=col_names, skiprows=skiprows, comment=comment, numeric_cols=numeric_cols)

    return gdf



'''
def get_gdf(filelist):
    gdf = None
    for file in filelist:
        if gdf is None:
            gdf = read_snex_csv(file, skiprows=0)
        else:
            gdf.append(read_snex_csv(file, skiprows=0))
    return gdf
'''


def main():

    parser = ArgumentParser()

    parser.add_argument('--file')

    args = parser.parse_args()

    gdf  = read_snex21_ts_summary_swe_v01(args.file)


if __name__ == '__main__':

    # file1="/home/jiangzhu/data/crrel/snex23_swe/SNEX23_SWE_Mar23IOP_AK_20230313_20230316_v01.0.csv"

    # file = "/home/jiangzhu/data/crrel/snowEx21_ts_snow_pits_v1/302507681/SNEX21_TS_SP_Summary_SWE_v01.csv"

    main()

    print("completed ...")













#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from pathlib import Path

import numpy as np
import pandas as pd
import xarray as xr

import matplotlib.pyplot as plt

from scipy.stats import pearsonr
from sklearn.metrics import mean_squared_error, mean_absolute_error

from spicy_snow.retrieval import retrieval_from_parameters


def bias(x, y): return np.mean(x - y)

def get_stats(x, y, nrmse = False):
    if type(x) == xr.DataArray: x = x.values.ravel()
    if type(y) == xr.DataArray: y = y.values.ravel()
    if type(x) == list: x = np.array(x)
    if type(y) == list: y = np.array(y)
    idx = (~np.isnan(x)) & (~np.isnan(y))
    x, y = x[idx], y[idx]
    r, p = pearsonr(x, y)
    b = bias(x, y)
    mae = mean_absolute_error(x, y)

    # rmse = mean_squared_error(x, y, squared = False)
    rmse = mean_squared_error(x, y)

    if nrmse:
        nrmse_value = rmse / np.mean(x)
        return r, b, mae, rmse, nrmse_value

    return r, b, mae, rmse

from scipy.stats import norm
def fischerz(truth, x1, x2):
    idx1 = (~np.isnan(truth)) & (~np.isnan(x1))
    idx2 = (~np.isnan(truth)) & (~np.isnan(x2))
    n = np.min([len(x1[idx1]), len(x2[idx2])])
    cor1 = pearsonr(truth[idx1], x1[idx1]).statistic
    cor2 = pearsonr(truth[idx2], x2[idx2]).statistic
    fischer1 = 0.5*np.log((1+cor1)/(1-cor1))
    fischer2 = 0.5*np.log((1+cor2)/(1-cor2))
    expected_sd = np.sqrt(1/(n-3))
    return 2 * (1 - norm(0, expected_sd).cdf(np.abs(fischer1 - fischer2)))


# In[ ]:


# npy_dirs = Path('/bsuhome/zacharykeskinen/scratch/spicy/param_npys')
# all_res = xr.open_dataset(npy_dirs.joinpath('grouped.nc'))
# all_res_dry = xr.open_dataset(npy_dirs.joinpath('dry_grouped.nc'))
# A = all_res.max('B').min('C')['pearsonr'].idxmax('A')
# B = all_res['pearsonr'].max('A').min('C').idxmax('B')
# print(A)
# print(B)
# print(all_res['mae'].sel(A = A, B = B).idxmin('C'))


# new table with WUS

# In[ ]:

# statscvsfile = '/home/jiangzhu/data/crrel/lidar_via_zach/scratch/spicy/SnowEx-Data-lave/stats_new.csv'

def stats_table_new(in_dir, statscvsfile):

    res = pd.DataFrame()

    dss = {fp.stem: xr.open_dataset(fp, decode_coords="all") for fp in in_dir.glob('*.nc')}
    mean_all = 0
    xs, ys = [], []
    dry_xs, dry_ys = [], []
    for stem, full_ds in dss.items():

        if stem == 'Frasier_2020-02-11':
            lidar_time = pd.to_datetime('2020-02-16')
            im_time = pd.to_datetime('2020-02-16')
        else:
            lidar_time = pd.to_datetime(full_ds.attrs['lidar-flight-time'])
            # drop the data at the sel_time
            full_ds_tmp = full_ds.drop_sel(time=lidar_time)
            im_time = pd.to_datetime(full_ds_tmp.sel(time = lidar_time, method = 'nearest').time.values.ravel()[0])

        d_days = im_time - lidar_time
        site_name = stem.replace('_', ' ').replace('Frasier', 'Fraser').split('-')[0]


        ds_img = full_ds.sel(time = im_time, method = 'nearest')
        ds_lidar = full_ds.sel(time = lidar_time, method = 'nearest')

        mean_all += ds_img['snow_depth'].mean()

        full_r, full_b, full_mae, full_rmse, full_nrmse = get_stats(ds_img['snow_depth'], ds_lidar['lidar-sd'], nrmse = True)
        # add RMSE and bias @ 90m
        for name, var in zip(['RMSE', 'Pearson R', 'nRMSE', 'Bias'], [full_rmse, full_r, full_nrmse, full_b]):
            res.loc[site_name, name] = var
        ys.extend(ds_lidar['lidar-sd'].data.ravel())
        xs.extend(ds_img['snow_depth'].data.ravel())

        # for dry only
        idx = ds_img['wet_snow'] == 0
        r, b, mae, rmse, nrmse  = get_stats(ds_img['snow_depth'].where(idx), ds_lidar['lidar-sd'].where(idx), nrmse = True)
        for name, var in zip(['RMSE', 'Pearson R', 'Bias'], [rmse, r, b]):
            res.loc[site_name, name + ' (Dry)'] = var
        res.loc[site_name, 'dry Fischer'] = fischerz(ds_lidar['lidar-sd'].data.ravel(), ds_img['snow_depth'].data.ravel(), ds_img['snow_depth'].where(idx).data.ravel())
        dry_ys.extend(ds_lidar['lidar-sd'].where(idx).data.ravel())
        dry_xs.extend(ds_img['snow_depth'].where(idx).data.ravel())

        # WUS over Lievens Improvement RMSE
        # l_ds = retrieval_from_parameters(full_ds, A = lievens_params[0], B = lievens_params[1], C = lievens_params[2]).sel(time = im_date, method = 'nearest')
        # l_r, l_b, l_mae, l_rmse, l_nrmse = get_stats(l_ds['lidar-sd'], l_ds['snow_depth'], nrmse = True)
        # res.loc[site_name, 'L22 RMSE'] = l_rmse
        # res.loc[site_name, 'L22 R'] = l_r
        # res.loc[site_name, 'L22 Fischer'] = fischerz(l_ds['lidar-sd'].data.ravel(), l_ds['snow_depth'].data.ravel(), ds['snow_depth'].data.ravel())
        res.loc[site_name, 'mean-all-sd'] = ds_lidar['lidar-sd'].mean().data.ravel()[0]
        res.loc[site_name, 'mean-dry-sd'] = ds_lidar['lidar-sd'].where(idx).mean().data.ravel()[0]

    r, b, mae, rmse = get_stats(xs, ys)
    for name, var in zip(['RMSE', 'Pearson R', 'Bias'], [rmse, r, b]):
        # res.loc['All Sites', name] = all_res.sel(A = 1.5, B = 0.1, C = 0.60)[name.lower().replace(' ', '')]
        res.loc['All Sites', name] = var
    # res.loc['All Site', 'nRMSE'] = all_res.sel(A = 1.5, B = 0.1, C = 0.60)['rmse'] /(mean_all / (len(dss)))

    r, b, mae, rmse = get_stats(dry_xs, dry_ys)
    for name, var in zip(['RMSE', 'Pearson R', 'Bias'], [rmse, r, b]):
        # res.loc[site_name, name + ' (Dry)'] = all_res_dry.sel(A = 1.5, B = 0.1, C = 0.60)[name.lower().replace(' ', '')]
        res.loc['All Sites', name + ' (Dry)'] = var

    res.loc['All Sites', 'dry Fischer'] = fischerz(xr.DataArray(ys), xr.DataArray(xs), xr.DataArray(dry_xs))

    res.loc['All Sites', 'mean-all-sd'] = np.nanmean(ys)
    res.loc['All Sites', 'mean-dry-sd'] = np.nanmean(dry_ys)
    # res.loc['All Sites', 'L22 RMSE'] = all_res.sel(A = 1.5, B = 0.1, C = 0.60)['rmse'] - all_res.sel(A = lievens_params[0], B = lievens_params[1], C = lievens_params[2])['rmse']
    # res.loc['All Sites', 'L22 R'] = all_res.sel(A = 1.5, B = 0.1, C = 0.60)['pearsonr'] - all_res.sel(A = lievens_params[0], B = lievens_params[1], C = lievens_params[2])['pearsonr']


    # In[ ]:


    res = res.applymap(lambda x: f'{x:.2f}').reindex(['All Sites', 'Banner 2020', 'Banner 2021', 'Dry Creek 2020',\
         'Fraser 2020', 'Fraser 2021', 'Little Cottonwood 2021', 'Mores 2020', 'Mores 2021', 'Cameron 2021'])
    res.drop(['nRMSE'], axis = 1)

    res.to_csv(statscvsfile)


    # In[ ]:


    for r in res[['Bias', 'Bias (Dry)']].values:
        print(f'& {r[0]} & {r[1]}')

    return res


def stats_table_old(in_dir, statscsvfile):
    # # old table with spatial resolutions

    # In[ ]:


    res = pd.DataFrame()
    coarse_x = []
    coarse_y = []

    coarse_x_1k = []
    coarse_y_1k = []
    dss = {fp.stem: xr.open_dataset(fp) for fp in in_dir.glob('*.nc')}

    for stem, ds in dss.items():

        if stem == 'Frasier_2020-02-11':
            lidar_time = pd.to_datetime('2020-02-16')
            im_time = pd.to_datetime('2020-02-16')
            r, b, mae, rmse = get_stats(ds['lidar-sd'], ds['snow_depth'].sel(time = '2020-02-16'))


        else:
            lidar_time = pd.to_datetime(ds.attrs['lidar-flight-time'])
            # drop the data at the sel_time
            ds_tmp = ds.drop_sel(time=lidar_time)
            im_time = pd.to_datetime(ds_tmp.sel(time = lidar_time, method = 'nearest').time.values.ravel()[0])

            ds_lidar =  ds.sel(time = lidar_time, method = 'nearest')
            ds_im = ds.sel(time = im_time, method = 'nearest')

            r, b, mae, rmse = get_stats(ds_lidar['lidar_sd'], ds_im['snow_depth'])

        d_days = im_time - lidar_time

        site_name = stem.replace('_', ' ').split('-')[0]

        # add RMSE and bias @ 90m
        for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
            res.loc[site_name, name] = var

        # for dry only
        idx = ds['wet_snow'].sel(time = im_time, method = 'nearest') == 0
        r, b, mae, rmse  = get_stats(ds['lidar-sd'].where(idx), ds['snow_depth'].sel(time = im_time, method = 'nearest').where(idx))
        for name, var in zip(['RMSE'], [rmse]):
            res.loc[site_name, name + ' (Dry)'] = var

        # @ 300 m
        ds_500 = ds.coarsen(x = 3, y = 3, boundary = 'pad').mean()
        r, b, mae, rmse  = get_stats(ds_500['lidar-sd'], ds_500['snow_depth'].sel(time = im_time, method = 'nearest'))
        coarse_x.append(ds_500['lidar-sd'].values.ravel())
        coarse_y.append(ds_500['snow_depth'].sel(time = im_time, method = 'nearest').values.ravel())

        for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
            res.loc[site_name, name+' @ 300m'] = var

        # @ 500 m
        ds_500 = ds.coarsen(x = 6, y = 6, boundary = 'pad').mean()
        r, b, mae, rmse  = get_stats(ds_500['lidar-sd'], ds_500['snow_depth'].sel(time = im_time, method = 'nearest'))
        coarse_x.append(ds_500['lidar-sd'].values.ravel())
        coarse_y.append(ds_500['snow_depth'].sel(time = im_time, method = 'nearest').values.ravel())

        for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
            res.loc[site_name, name+' @ 500m'] = var

        # @ 1 km

        ds_500 = ds.coarsen(x = 11, y = 11, boundary = 'pad').mean()
        r, b, mae, rmse  = get_stats(ds_500['lidar-sd'], ds_500['snow_depth'].sel(time = im_time, method = 'nearest'))
        coarse_x_1k.append(ds_500['lidar-sd'].values.ravel())
        coarse_y_1k.append(ds_500['snow_depth'].sel(time = im_time, method = 'nearest').values.ravel())

        for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
            res.loc[site_name, name+' @ 1km'] = var

    '''
    for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
        res.loc['All Sites', name] = all_res.sel(A = 1.5, B = 0.1, C = 0.59)[name.lower().replace(' ', '')]
    '''

    r, b, mae, rmse  = get_stats(np.concatenate(coarse_x).ravel(), np.concatenate(coarse_y).ravel())
    for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
        res.loc['All Sites', name+' @ 500m'] = var

    r, b, mae, rmse  = get_stats(np.concatenate(coarse_x_1k).ravel(), np.concatenate(coarse_y_1k).ravel())
    for name, var in zip(['RMSE', 'Pearson R'], [rmse, r]):
        res.loc['All Sites', name+' @ 1km'] = var

    res.to_csv(statscsvfile)

    return res

# In[ ]:

def main():
    lievens_params = [2, 0.5, 0.44]
    # wus_params = [2.5, 0.2, 0.55]
    # wus_v2_params = [1.5, 0.1, 0.59]

    in_dir = Path('/home/jiangzhu/data/crrel/lidar2/scratch').expanduser().resolve()

    data_dir = Path('/home/jiangzhu/data/crrel/lidar2/scratch').expanduser().resolve()

    csvfile_new = '/home/jiangzhu/data/crrel/lidar2/stats_table_new.csv'

    csvfile_old = '/home/jiangzhu/data/crrel/lidar2/stats_table_old.csv'

    res_new = stats_table_new(in_dir, csvfile_new)


    # res_old = stats_table_old(in_dir, csvfile_old), not work yet, need debug it

if __name__ == '__main__':

    main()





from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser
import geopandas as gpd
import pandas as pd
from pathlib import Path
from asf_snow.utils.analyze_sentinel1_snow import draw_combined_gdf, draw_histgram, draw_scatter_plot



filelist = [
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20201101_1130/sentinel1_20201101_1130_500_24h_close_boise_river_basin.shp',
    '//media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boiseboise_20201201_1231/sentinel1_20201201_1231_500_24h_close_boise_river_basin.shp',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210101_0131/sentinel1_20210101_0131_500_24h_close_boise_river_basin.shp',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210201_0228/sentinel1_20210201_0228_500_24h_close_boise_river_basin.shp',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210301_0331/sentinel1_20210301_0331_500_24h_close_boise_river_basin.shp',
    '/media/jiangzhu/Elements/crrel/sentinel1_snowEx21_ts_snow_pits_v1/boise/boise_20210401_0430/sentinel1_20210401_0430_500_24h_close_boise_river_basin.shp'
    ]


def combine_multiple_statistic_results(filelist:list, timethresh=None):

    file = filelist[0]
    out_dir = str(Path(file).parent.parent)
    lst = Path(file).stem.split("_")

    res = int(lst[3])
    timeint = int(lst[4][:-1])
    timethresh = int(lst[5][:-1])
    base = lst[6]
    method = lst[7]
    precip = lst[8]
    outfile = f'{out_dir}/sentinel1_202011_202104_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_boise_river_basin.png'

    num = len(filelist)
    tot = None
    for i in range(num):

        if Path(filelist[i]).is_file():
            gdf = gpd.read_file(filelist[i])
            if tot is None:
                tot = gdf.copy(deep=True)
            else:
                tot = pd.concat([tot, gdf], ignore_index=True)

    # if tot is not None:
    #    draw_combined_gdf(tot, res=res, timeint=timeint, timethresh=timethresh, base=base, method=method,
    #                      outfile=outfile, precip=precip)

    # draw_scatter_plot(tot, res=res)

    # calculate the correlation between snow depth and observation

    return tot


def read_filelist(filelistname):
    filelst = []

    with open(filelistname) as f:
        while True:
            line = f.readline()
            if line:
                filelst.append(line.strip())
            else:
                break

    return filelst


def draw_snow_difference_histgram(filelistname):
    # read the file list
    filelist = read_filelist(filelistname)
    file = filelist[0]
    out_dir = str(Path(file).parent.parent)
    lst = Path(file).stem.split("_")
    res = int(lst[3])
    timeint = int(lst[4][:-1])
    timethresh = int(lst[5][:-1])
    base = lst[6]
    method = lst[7]
    precip = lst[8]
    outfile = f'{out_dir}/histgram_snow_depth_diff_{res}_{timeint}h_{timethresh}h_{base}_{method}_{precip}_boise.png'

    # combine the statistic
    combined = combine_multiple_statistic_results(filelist)

    # draw histgram of snow depth difference

    draw_histgram(combined, res, timeint, timethresh, base, method, outfile, precip)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('--file', type=str,  required=True)

    args = parser.parse_args()
    filelst = []

    with open(args.file) as f:
        while True:
            line = f.readline()
            if line:
                filelst.append(line.strip())
            else:
                break

    combine_multiple_statistic_results(filelst)

    print("completed ...")